// Generated by CoffeeScript 1.6.3
var CancellationToken, Client, defaultOptions, exports, request, _,
  __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

request = require('request');

_ = require('underscore');

defaultOptions = {
  pool: {
    maxSockets: 100
  },
  followAllRedirects: true
};

CancellationToken = (function() {
  function CancellationToken(hosts) {
    this.hosts = hosts;
    this.abort = __bind(this.abort, this);
    this.isAborted = __bind(this.isAborted, this);
    this.setRequest = __bind(this.setRequest, this);
    this.aborted = false;
  }

  CancellationToken.prototype.setRequest = function(req) {
    return this.req = req;
  };

  CancellationToken.prototype.isAborted = function() {
    return this.aborted;
  };

  CancellationToken.prototype.abort = function() {
    this.aborted = true;
    if (this.req != null) {
      return this.req.abort();
    }
  };

  CancellationToken.prototype.cancel = CancellationToken.prototype.abort;

  return CancellationToken;

})();

Client = (function() {
  function Client(hosts, sslopts) {
    this.hosts = hosts;
    this.sslopts = sslopts;
    this._multiserverHelper = __bind(this._multiserverHelper, this);
    this["delete"] = __bind(this["delete"], this);
    this.patch = __bind(this.patch, this);
    this.post = __bind(this.post, this);
    this.get = __bind(this.get, this);
    this.put = __bind(this.put, this);
    this.execute = __bind(this.execute, this);
  }

  Client.prototype.execute = function(method, options, callback) {
    var errors, opt, servers, token;
    opt = _.defaults(_.clone(options), defaultOptions, {
      method: method
    });
    servers = _.shuffle(this.hosts);
    token = new CancellationToken(servers);
    this._multiserverHelper(servers, opt, token, errors = [], callback);
    return token;
  };

  Client.prototype.put = function(options, callback) {
    return this.execute("PUT", options, callback);
  };

  Client.prototype.get = function(options, callback) {
    return this.execute("GET", options, callback);
  };

  Client.prototype.post = function(options, callback) {
    return this.execute("POST", options, callback);
  };

  Client.prototype.patch = function(options, callback) {
    return this.execute("PATCH", options, callback);
  };

  Client.prototype["delete"] = function(options, callback) {
    return this.execute("DELETE", options, callback);
  };

  Client.prototype._multiserverHelper = function(servers, options, token, errors, callback) {
    var error, host,
      _this = this;
    host = _.first(servers);
    options.url = "" + options.serverprotocol + "://" + host + options.path;
    if (token.isAborted()) {
      return;
    }
    if (host == null) {
      error = new Error('All servers returned error');
      error.errors = errors;
      return callback(error);
    }
    return token.setRequest(request(options, function(err, resp, body) {
      if (_this._isHttpError(err, resp)) {
        errors.push({
          server: host,
          httperror: err,
          response: resp
        });
        return _this._multiserverHelper(_.rest(servers), options, token, errors, callback);
      } else if (!token.isAborted()) {
        return _this._handleResponse(err, resp, body, callback);
      }
    }));
  };

  Client.prototype._isHttpError = function(err, resp) {
    return err || (((resp != null ? resp.statusCode : void 0) != null) && resp.statusCode >= 500);
  };

  Client.prototype._handleResponse = function(err, resp, body, callback) {
    var error;
    if (callback == null) {
      return;
    }
    if ((body != null ? body.errorCode : void 0) != null) {
      error = new Error((body != null ? body.message : void 0) || 'Etcd error');
      error.errorCode = body.errorCode;
      error.error = body;
      return callback(error, "", (resp != null ? resp.headers : void 0) || {});
    } else {
      return callback(null, body, (resp != null ? resp.headers : void 0) || {});
    }
  };

  return Client;

})();

exports = module.exports = Client;
